//
//  SearchTextField.m
//  SearchTextField
//
//  Created by Prachi Gauriar on Tue Mar 25 2003.
//  Copyright (c) 2003 Prachi Gauriar. All rights reserved.
//

#import "STFPopUpButtonCell.h"
#import "SearchTextFieldCell.h"
#import "SearchTextField.h"

@implementation SearchTextField

+ (Class)cellClass
{
    return [SearchTextFieldCell class];
}


- (void)awakeFromNib
{
    [self setCell:[[SearchTextFieldCell alloc] initTextCell:@""]];

    searchesIncrementally = NO;
    
    return;
}


- (void)mouseDown:(NSEvent *)theEvent
{
    NSPoint pointInField = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    NSRect cellFrame = [self convertRect:[self frame] fromView:[self superview]];

    if (NSMouseInRect(pointInField, [[self cell] cancelButtonRectFromRect:cellFrame], NO)) {
        [[self cell] cancelButtonClickedWithFrame:cellFrame inView:self];
    } else if (NSMouseInRect(pointInField, [[self cell] popUpButtonRectFromRect:cellFrame], NO)) {
        [[self cell] popUpButtonClickedWithFrame:cellFrame inView:self];
    }

    return;
}


- (void)resetCursorRects
{
    return;
}


- (BOOL)isFirstResponder
{
    NSResponder *first = ([self window] != nil) ? [[self window] firstResponder] : nil;

    while (first != nil && [first isKindOfClass:[NSView class]]) {
        first = [first superview];

        if (first == self) {
            return YES;
        }
    }

    return NO;
}


- (BOOL)becomeFirstResponder
{
    if ([[self textColor] isEqualTo:[NSColor disabledControlTextColor]]) {
        [self setTextColor:[NSColor controlTextColor]];
        [self setStringValue:@""];
    }

    return [super becomeFirstResponder];
}


- (BOOL)searchesIncrementally
{
    return searchesIncrementally;
}


- (void)setSearchesIncrementally:(BOOL)aBoolean
{
    searchesIncrementally = aBoolean;
}


- (void)textDidChange:(NSNotification *)aNotification
{
    if (searchesIncrementally) {
        [[self cell] searchSubmittedFromView:self];
    }

    return;
}


- (NSString *)titleOfSelectedPopUpItem
{
    return [[[self cell] popUpButtonCell]  titleOfSelectedItem];
}



- (void)textDidEndEditing:(NSNotification *)aNotification
{
    BOOL showGreyText = YES;
    NSMutableDictionary *userInfo;
    id textMovement;
    NSFormatter *formatter;
    NSString *string;
    id cell = [self cell];
    NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];

    formatter = [cell formatter];

    string = [[[[[self window] fieldEditor:YES forObject:self] string] copy] autorelease];


    if (formatter == nil) {
        [cell setStringValue:string];
    } else {
        // If we have a special formatter, transform our string based on that method
        id newObjectValue;
        NSString *error;
        SEL controlFailedMethod = @selector(control:didFailToFormatString:errorDescription:);

        if ([formatter getObjectValue:&newObjectValue forString:string errorDescription:&error]) {
            [cell setObjectValue:newObjectValue];
        } else if ([[self delegate] respondsToSelector:controlFailedMethod] &&
                   [[self delegate] control:self didFailToFormatString:string
                           errorDescription:error]) {
            [cell setStringValue:string];
        }
    }

    [cell endEditing:[aNotification object]];

    // Get information from the notifcation dictionary so that we can decide
    // what to do in response to the user's action
    userInfo = [[NSMutableDictionary alloc] initWithDictionary:[aNotification userInfo]];
    [userInfo setObject:[aNotification object] forKey:@"NSFieldEditor"];
    [defaultCenter postNotificationName:NSControlTextDidEndEditingNotification
                                 object:self
                               userInfo:userInfo];
    textMovement = [[aNotification userInfo] objectForKey:@"NSTextMovement"];

    if (textMovement) {
        switch ([(NSNumber *)textMovement intValue]) {
            // If the user hit enter, send our action to our target
            case NSReturnTextMovement:
                [[self cell] searchSubmittedFromView:self];

                if ([self sendAction:[self action] to:[self target]] == NO) {
                    NSEvent *event = [[self window] currentEvent];

                    if ([self performKeyEquivalent:event] == NO &&
                        [[self window] performKeyEquivalent:event] == NO) {
                        [self selectText:self];
                    }
                } break;

                // If the user hit tab, go to the next key view
            case NSTabTextMovement:
                [[self window] selectKeyViewFollowingView:self];

                if ([[self window] firstResponder] == [self window]) {
                    [self selectText:self];
                } break;

                // If the user hit shift-tab, go to the key view before us
            case NSBacktabTextMovement:
                [[self window] selectKeyViewPrecedingView:self];

                if ([[self window] firstResponder] == [self window]) {
                    [self selectText:self];
                } break;
            default:
                if ([self isFirstResponder]) {
                    showGreyText = NO;
                }
        }
    }

    [cell showSelectedPopUpItem:showGreyText];

    return;
}


- (void)addPopUpMenuItemsWithTitles:(NSArray *)itemTitles
{
    [[[self cell] popUpButtonCell] addItemsWithTitles:itemTitles];

    return;
}


- (IBAction)makeSmall:(id)sender
{
    [[self cell] setControlSize:NSSmallControlSize];
    NSLog(@"Control size = %@\n", [[self cell] controlSize] == NSSmallControlSize ? @"NSSmallControllSize" : @"NSRegularControlSize");
}


@end
